/* ====================================================================
   Copyright 2002-2004   Apache Software Foundation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */

package org.apache.poi.hslf.util;

import org.apache.poi.util.LittleEndian;
import org.apache.poi.hslf.record.RecordTypes;
import org.apache.poi.poifs.filesystem.*;
import org.apache.poi.ddf.*;

import java.io.*;
import java.util.List;
import java.util.Iterator;

/**
 * Utility class which dumps the structure of a powerpoint file into XML
 *
 * @author Yegor Kozlov
 */

public class PPDump {
    public static final int HEADER_SIZE = 8;
    public final static String PPDOC_ENTRY = "PowerPoint Document";
    public static String CR = System.getProperty("line.separator");

    protected Writer out;
    protected byte[] docstream;

    public PPDump(File ppt) throws IOException {
        FileInputStream fis = new FileInputStream(ppt);
        POIFSFileSystem fs = new POIFSFileSystem(fis);
        fis.close();

        //read the document entry from OLE file system
        DocumentEntry entry = (DocumentEntry)fs.getRoot().getEntry(PPDOC_ENTRY);
        docstream = new byte[entry.getSize()];
        DocumentInputStream is = fs.createDocumentInputStream(PPDOC_ENTRY);
        is.read(docstream);

    }

    /**
     * Dump the structure of the supplied PPT file into XML
     * @param out <code>Writer</code> to write out
     * @throws IOException
     */
    public void dump(Writer out) throws IOException {
        this.out = out;
        int padding  = 0;
        int offset = 0;
        EscherRecordFactory ef = new DefaultEscherRecordFactory();

        write(out, "<Presentation>" + CR, padding);
        while (offset < docstream.length){
            EscherRecord rec = ef.createRecord(docstream, offset);
            rec.fillFields( docstream, offset, ef );
            offset += rec.getRecordSize();
            padding++;
            dump(rec, padding);
            padding--;
        }
        write(out, "</Presentation>", padding);
    }

    /**
     * Dumps EscherRecord into XML format
     * @param rec EscherRecord to dump
     * @param padding
     * @throws IOException
     */
    public void dump( EscherRecord rec, int padding) throws IOException {
        int type = rec.getRecordId();
        if (type < 0){ //type is unsigned short, convert if it is required
            type = (((int) Short.MAX_VALUE + 1) * 2 + type);
        }
        String recname = RecordTypes.recordName(type);
        write(out, "<"+recname + " info=\""+rec.getOptions()+"\" type=\""+type
                +"\" size=\""+(rec.getRecordSize()-8)+"\">" + CR, padding);

        //hack. EscherTextboxRecord is container but it doesn't process child records
        if (type == RecordTypes.EscherClientTextbox){
            byte[] recdata = rec.serialize();
            rec = new EscherContainerRecord();
            rec.fillFields(recdata, 0, new DefaultEscherRecordFactory());
        }

        if (rec.isContainerRecord()){
            padding++;
            List lst = rec.getChildRecords();
            for (Iterator it = lst.iterator(); it.hasNext();){
                EscherRecord r = (EscherRecord)it.next();
                dump(r, padding);
            }
            padding--;
        } else {
            byte[] recdata = rec.serialize();
            //dump first 100 bytes of the atom data excluding record header
            dump(out, recdata, 8, Math.min((recdata.length-8), 100), padding, 25);
        }
        write(out, "</"+recname + ">" + CR, padding);
    }

    public static void main(String[] args) throws Exception {
        if (args.length == 0){
            System.out.println("Usage: PPDump <pptfile>");
            return;
        }
        for (int i = 0; i < args.length; i++){
            PPDump dump = new PPDump(new File(args[i]));

            System.out.println("Dumping " + args[i]);
            FileWriter out = new FileWriter("es_" + args[i] + ".xml");
            dump.dump(out);
            out.close();
        }
    }

    private static void write(Writer out, String str, int padding) throws IOException {
        for (int i = 0; i < padding; i++) out.write("  ");
        out.write(str);
    }

    /**
     *  dump binary data to <code>out</code> with the specified padding
     */
    private static void dump(Writer out, byte[] data, int offset, int length, int padding, int linesize) throws IOException {
        for (int i = 0; i < padding; i++) out.write("  ");
        int i;
        for (i = offset; i < (offset + length); i++) {
            int c = data[i];
            out.write((char) hexval[(c & 0xF0) >> 4]);
            out.write((char) hexval[(c & 0x0F) >> 0]);
            out.write(' ');
            if((i+1-offset) % linesize == 0 && i != (data.length-1)) {
                out.write(CR);
                for (int j = 0; j < padding; j++) out.write("  ");
            }
        }
        out.write(CR);
    }

    private static final byte hexval[] =
        {(byte) '0', (byte) '1', (byte) '2', (byte) '3',
         (byte) '4', (byte) '5', (byte) '6', (byte) '7',
         (byte) '8', (byte) '9', (byte) 'A', (byte) 'B',
         (byte) 'C', (byte) 'D', (byte) 'E', (byte) 'F'};



}
